/*
*****************************************************************************
* Copyright(c) 2014 NXP Semiconductors                                      *
* All rights are reserved.                                                  *
*                                                                           *
* <TODO PUT LICENSE HEADER HERE>                                            *
*                                                                           *
*****************************************************************************
*/

/***********************************************************************/
/* INCLUDES                                                            */
/***********************************************************************/
#include <board.h>
#include "unity_fixture.h"
#include "HAL_timer_driver.h"
#include "../isr_driver_intern.h"

/***********************************************************************/
/* DEFINES                                                             */
/***********************************************************************/
#define GPIOINT_CHANNEL_MIN 0
#define GPIOINT_NUMBER_CHANNELS 8
/***********************************************************************/
/* LOCAL VARIABLES                                                     */
/***********************************************************************/
uint32_t value;

/***********************************************************************/
/* LOCAL FUNCTION PROTOTYPES                                           */
/***********************************************************************/
void test_handler0(ISR_ARGUMENT_T arg);

/***********************************************************************/
/* TEST GROUP DEFINITION                                               */
/***********************************************************************/
TEST_GROUP(isr_driver);

TEST_SETUP(isr_driver)
{
	HAL_ISR_Init();
}

TEST_TEAR_DOWN(isr_driver)
{
}

TEST_GROUP_RUNNER(isr_driver)
{
	RUN_TEST_CASE(isr_driver, callback_list_is_valid);
	RUN_TEST_CASE(isr_driver, registered_isr_is_called);
	RUN_TEST_CASE(isr_driver, callback_list_isr_is_called);
	RUN_TEST_CASE(isr_driver, interrupt_disabled_after_trigger);
	RUN_TEST_CASE(isr_driver, sleep_times_out_if_not_triggered);
	RUN_TEST_CASE(isr_driver, sleep_returns_if_triggered);
	RUN_TEST_CASE(isr_driver, wait_with_null_handler_possible);
}

/***********************************************************************/
/* TESTS                                                               */
/***********************************************************************/
TEST(isr_driver, callback_list_is_valid)
{
	size_t i = 0;
	for(i = 0; i < ISR_MAX_CALLBACKS; i++)
	{
		SELECT_CALLBACK(i);
		TEST_ASSERT_EQUAL(ISR_INVALID_HANDLER, callback->handler);
		TEST_ASSERT_UINT_WITHIN(GPIO_NUMBER_PINS, GPIO_PIN_MIN, callback->pin_func);
		TEST_ASSERT_UINT_WITHIN(GPIOINT_NUMBER_CHANNELS, GPIOINT_CHANNEL_MIN, callback->channel);
	}
}

TEST(isr_driver, registered_isr_is_called)
{
	SELECT_CALLBACK(0);

	/* setup */
	value = 0;

	/* test */
	HAL_ISR_RegisterCallback(0, ISR_EDGE_RISE, &test_handler0, NULL);

	/* trigger interrupt manually */
	NVIC_SetPendingIRQ(GPIO_PININT_NVIC(callback->channel));

	/* check */
	HAL_Timer_delay_ms(1);
	TEST_ASSERT_EQUAL_HEX32(0xFEEDCAFE, value);
}

TEST(isr_driver, callback_list_isr_is_called)
{
	volatile size_t i = 0;

	/* register callbacks */
	for( i = 0; i < ISR_MAX_CALLBACKS; i++)
	{
		/* test */
		HAL_ISR_RegisterCallback(i, ISR_EDGE_RISE, &test_handler0, NULL);
	}

	/* test */
	for( i = ISR_MAX_CALLBACKS; i > 0; i--)
	{
		SELECT_CALLBACK(i-1);

		/* setup */
		value = 0;

		/* trigger interrupt manually */
		NVIC_SetPendingIRQ(GPIO_PININT_NVIC(callback->channel));

		/* check */
		HAL_Timer_delay_ms(1);
		TEST_ASSERT_EQUAL_HEX32(0xFEEDCAFE, value);
	}
}

TEST(isr_driver, interrupt_disabled_after_trigger)
{
	SELECT_CALLBACK(0);

	/* register */
	HAL_ISR_RegisterCallback(0, ISR_EDGE_RISE, &test_handler0, NULL);

	/* trigger */
	value = 0;
	NVIC_SetPendingIRQ(GPIO_PININT_NVIC(callback->channel));

	/* check */
	HAL_Timer_delay_ms(1);
	TEST_ASSERT_EQUAL_HEX32(0xFEEDCAFE, value);

	/* trigger */
	value = 0;
	NVIC_SetPendingIRQ(GPIO_PININT_NVIC(callback->channel));

	/* check */
	HAL_Timer_delay_ms(1);
	TEST_ASSERT_EQUAL_HEX32(0x00000000, value);
}

TEST(isr_driver, sleep_times_out_if_not_triggered)
{
	BOOL retval;

	/* register and trigger */
	HAL_ISR_RegisterCallback(0, ISR_EDGE_RISE, &test_handler0, NULL);

	retval = HAL_ISR_SleepWithTimeout(0, 5);

	TEST_ASSERT_TRUE(retval);
}

TEST(isr_driver, sleep_returns_if_triggered)
{
	BOOL retval;

	SELECT_CALLBACK(0);

	/* register and trigger */
	HAL_ISR_RegisterCallback(0, ISR_EDGE_RISE, &test_handler0, NULL);

	/* trigger */
	NVIC_SetPendingIRQ(GPIO_PININT_NVIC(callback->channel));

	retval = HAL_ISR_SleepWithTimeout(0, 5);

	TEST_ASSERT_FALSE(retval);
}

TEST(isr_driver, wait_with_null_handler_possible)
{
	BOOL retval;

	SELECT_CALLBACK(0);

	/* register and trigger */
	HAL_ISR_RegisterCallback(0, ISR_EDGE_RISE, NULL, NULL);

	/* trigger */
	NVIC_SetPendingIRQ(GPIO_PININT_NVIC(callback->channel));

	retval = HAL_ISR_SleepWithTimeout(0, 5);

	TEST_ASSERT_FALSE(retval);
}
/***********************************************************************/
/* LOCAL FUNCTIONS                                                     */
/***********************************************************************/
void test_handler0(ISR_ARGUMENT_T arg)
{
	value = 0xFEEDCAFE;
}
